unit ScrollLabelUnit;


{
 ScrollLabelUnit made by Julian Meyn (lazarusjulian) February 2020
 Feel free to use, modify etc. but please keep this note here. Thank you!
 This is the third lazarus component I have ever made, forgive me bad programming style etc. ;)

 Usage:
 Set Text of the Scrolllabel via LabelText(Object Inspector) or
 setText(sourcecode) at any time, even while scrolling.
 If You use only one Scrollabel, you should enable automatic Mode.
 Hint: OnIterationEnd is not called in automatic Mode!!
 If you use two or more labels grouped above each other, you should disable
 automatic mode and call startScrollIteration of the next label in OnIterationEnd
 of the label which is above the next one.
 startScrollIteration of the first Label has to be called manually on formcreate
 or so to start the loop of scrollinglabels calling each other.

 To adjust scrolling speed, set the ScrollTimerInterval.
 lower interval = faster scrolling!
 To adjust the time the text spends at the borders, set TextWaitIntervalCount.
 The text waits at the border as many timer intervals as you have set.
 If the text is so short is does not have to be scrolled, the label will delay
 and wait noScrollDelayIterationCount of timer intervals before OnIterationEnd
 is called. The  noScrollDelayIterationCount should never be zero if you use
 multiple labels and set the text dynamically!
 -> infinite loop with no delay will block main thread

 Have fun!

}

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, LResources, Forms, Controls, Graphics, Dialogs, StdCtrls, ExtCtrls;

type
  TScrollLabel = class(TPanel)
  private
    automaticMode:boolean;
    scrollawayDone:boolean;
    scrollbackDone:boolean;
    foniterationend:TNotifyEvent;
    FStay:boolean;
    FnoScrollDelayIterationCount:integer;
    FTimer:TTimer;
    FStayCount:integer;
    FStayTime:integer;
    nothingTODOcounter:integer;
    FLabel:TLabel;
    procedure OnTimer(Sender:TObject);
  protected

  public
    procedure startScrollIteration;
    constructor Create(AOwner: TComponent); override;
    function getText:string;
    procedure setText(aText:string);
    function getTimerInterval:integer;
    procedure setTimerInterval(int:integer);
    function getFontSize:integer;
    procedure setFontSize(size:integer);
    function getFontColor:TColor;
    procedure setFontColor(Acolor:TColor);


  published
    property automatikMode: boolean read automaticMode write automaticMode;
    property ScrollTimerInterval:integer read getTimerInterval write setTimerInterval;
    property TextWaitIntervalCount: integer read FStayTime write FStayTime;
    property OnIterationEnd:TNotifyEvent read foniterationend write foniterationend;
    property noScrollDelayIterationCount: integer read FnoScrollDelayIterationCount write FnoScrollDelayIterationCount;
    property FontSize:integer read getFontSize write setFontSize;
    property FontColor:TColor read getFontColor write setFontColor;
    property LabelText:string read getText write setText;
  end;

procedure Register;

var ScrollLabel:TScrollLabel;

implementation

function TScrollLabel.getFontColor:TColor;
begin
  result := FLabel.Font.Color;
end;

procedure TScrollLabel.setFontColor(aColor:TColor);
begin
  Flabel.Font.Color:=acolor;
end;

function TScrollLabel.getFontSize:integer;
begin
  result := FLabel.Font.Size;
end;

procedure TScrollLabel.setFontSize(size:integer);
begin
  FLabel.font.Size := size;
end;

procedure TScrollLabel.startScrollIteration;
begin
  scrollawayDone:=false;
  scrollbackDone:=false;
  FLabel.Left := 0;
  nothingTODOcounter := 0;
  Ftimer.enabled := true;
end;


function TScrollLabel.getText:string;
begin
  result := FLabel.caption;
end;


procedure TScrollLabel.setText(aText:string);
begin
  FLabel.caption := aText;
  Flabel.left := 0;
  if automaticMode then Ftimer.enabled := true;
end;

function TScrollLabel.getTimerInterval:integer;
begin
  result := Ftimer.interval;
end;


procedure TScrollLabel.setTimerInterval(int:integer);
begin
  Ftimer.interval:=int;
end;

procedure TScrollLabel.OnTimer(Sender:TObject);
begin
 // oniterationend(self);

  if FLabel.Width > ScrollLabel.Width then
  begin   //das Element muss gescrollt werden
    if not FStay then  //warten oder nicht?
    begin
      if not scrollawayDone then
      begin //nach links scrollen, wenn Grenze erreicht Richtung ändern
          if FLabel.Left <= (ScrollLabel.Width-FLabel.Width) then
          begin
            FStayCount := 0;
            scrollawayDone := true;
            FStay := true;
          end else FLabel.Left := FLabel.left-1;
      end
      else
      begin //nach rechts scrollen, wenn Grenze erreicht Richtung ändern
        if (FLabel.Left >= 0) then
        begin
          FStayCount := 0;
          scrollbackDone := true;
          FStay := false;
        end else FLabel.Left := FLabel.left+1;
      end;
    end
    else
    begin //weiter warten oder nicht?
      if FStayCount < FStayTime then inc(FStayCount)
      else
      begin
        FStay := false;
        FStayCount := 0;
      end;
    end;
  end
  else
  begin
    nothingTODOcounter := nothingTODOcounter+1;
    if nothingTODOcounter >=FnoScrollDelayIterationCount then
    begin
      scrollawayDone := true;
      scrollbackDone:=  true;
      nothingTODOcounter := 0;
    end;
  end;

  if (scrollawayDone and scrollbackDone ) then
  begin
    if (not automaticMode) then
    begin
      Ftimer.enabled := false;
      oniterationend(self);
    end
    else
    begin
      Fstay := true;
      scrollawayDone:=false;
      scrollbackDone:=false;
    end
  end;

end;

constructor TScrollLabel.Create(AOwner: TComponent);
begin
  inherited create(AOWner);
  ScrollLabel := self;
  self.Caption:='';
  self.Height:=18;
  FLabel:= TLabel.Create(self);
  self.InsertControl(FLabel);

  FStay := false;
  FLabel.Align:=Alnone;
  FLabel.left := 0;
  FLabel.Top:=0;
  FLabel.Font.Color:=clblack;
  flabel.font.Size:=10;
  FLabel.Caption:='ScrollLabel';

  FStayTime:=166;
  automaticMode := false;
  scrollawayDone:=false;
  scrollbackDone:=false;

  FTimer := TTimer.create(self);
  Ftimer.enabled := false;
  FTimer.Interval := 25;
  FTimer.OnTimer := @OnTimer;

  FnoScrollDelayIterationCount := 100;



end;

procedure Register;
begin
  {$I scrolllabelunit_icon.lrs}
  RegisterComponents('Additional',[TScrollLabel]);
end;

end.
